@@ -79,8 +79,9 @@ class AppEngine:
         Stops and clean handlers
         """
         logger.info(__name__, f"Stopping app={self.app_key}...")
-        for event_name in self._running.keys():
-            await self.stop_event(event_name)
+        for event_name, running in self._running.items():
+            if running.locked():
+                await self.stop_event(event_name)
         if self.stream_manager:
             await asyncio.sleep(self.app_config.engine.read_stream_timeout + 5)
             await self.stream_manager.close()
@@ -529,6 +530,9 @@ class AppEngine:
         logger.info(__name__, "Finished service.", extra=extra(prefix='service.', **log_info))
         return last_result
 
+    def is_running(self, event_name) -> bool:
+        return self._running[event_name].locked()
+
     async def stop_event(self, event_name: str):
         """
         Sets running state to stopped for a continuous-running event.
@@ -538,6 +542,8 @@ class AppEngine:
         """
         if self._running[event_name].locked():
             self._running[event_name].release()
+        else:
+            raise RuntimeError(f"Cannot stop non running event: {event_name}.")
 
     @staticmethod
     def _config_effective_events(app_config: AppConfig) -> Dict[str, EventDescriptor]:
